# -*- coding: utf-8 -*-
import logging
from rest_framework.exceptions import NotFound
from rest_framework.response import Response
from rest_framework.serializers import Serializer

from yaphone.advisor.advisor.serializers.device import GsmSerializer, WifiSerializer, LocationSerializer
from yaphone.advisor.advisor.views.base import StatelessView
from yaphone.http_geobase_lookuper.httpgeobase import FED_SUBJECT_TYPE
from yaphone.utils import geo

logger = logging.getLogger(__name__)


# noinspection PyAbstractClass
class GetLocationSerializer(Serializer):
    location = LocationSerializer(required=False)
    cells = GsmSerializer(many=True, required=False)
    wifi_networks = WifiSerializer(many=True, required=False)


class GetLocationView(StatelessView):
    validator_class = GetLocationSerializer

    def get_lbs_location(self, data):
        lbs_position = geo.lbs_locator.locate(
            gsm_cells=data.get('cells', []),
            wifi_networks=data.get('wifi_networks', []),
            uuid=self.uuid,
            ip=self.ip
        )
        if not lbs_position or lbs_position['type'] == 'ip':
            raise NotFound(detail='Could not determine user location')
        return lbs_position['latitude'], lbs_position['longitude'], lbs_position['precision']

    def post(self, request, *args, **kwargs):
        data = self.get_validated_data(request.data)

        if data.get('location'):
            location = data['location']
            latitude, longitude, accuracy = location['latitude'], location['longitude'], 0.
        else:
            latitude, longitude, accuracy = self.get_lbs_location(data)

        region_id = geo.geobase_lookuper.get_region_id_by_location(latitude, longitude)
        region = geo.geobase_lookuper.get_region_by_id(region_id)

        return Response({
            'region': {
                'name': region['name'],
                'region_id': region_id,
                'subject_id': geo.get_parent_region_with_type(region_id, FED_SUBJECT_TYPE),
            },
            'position': {
                'lat': latitude,
                'lon': longitude,
                'accuracy': accuracy,
            },
        })
